SystemOrganization addCategory: #'SqueakExamples-Cacheing'!
SystemOrganization addCategory: #'SqueakExamples-MaresyDoats'!
SystemOrganization addCategory: #'SqueakExamples-Reflection'!
SystemOrganization addCategory: #'SqueakExamples-SelfSuper'!
SystemOrganization addCategory: #'SqueakExamples-SnakesAndLadders'!
SystemOrganization addCategory: #'SqueakExamples-PostOffice'!

TestCase subclass: #AnonymousObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Reflection'!

!AnonymousObjectTest methodsFor: 'running' stamp: 'on 4/3/2006 16:20'!
testCount
	"Transcript closeAllViews; open"
	| set anonClass |
	set _ Set new.
	self assert: set size = 0.

	anonClass _ Behavior new.
	anonClass superclass: Set;
		setFormat: Set format;
		methodDictionary: MethodDictionary new.
	
	anonClass compile: 'class ^ super class superclass'. "doesn't work since class is inlined"
	anonClass compile: 'add: anObject Transcript show: ''adding '', anObject printString; cr. ^ super add: anObject'.
	
	self assert: set class = Set.
	self assert: set size = 0.

	set add: 1.
	self assert: set size = 1.
	
	set primitiveChangeClassTo: anonClass basicNew.
	
	self assert: set class ~= Set.				":-("
	set add: 2.
	self assert: set size = 2.
! !

TestCase subclass: #MareTest
	instanceVariableNames: 'mare oats doe lamb kid ivy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!MareTest methodsFor: 'running' stamp: 'on 3/9/2006 14:02'!
setUp
	mare := Mare new.
	doe := Doe new.
	lamb := Lamb new.
	kid := Kid new.
	oats := Oats new.
	ivy := Ivy new.! !

!MareTest methodsFor: 'running' stamp: 'on 3/9/2006 14:06'!
testEating
	self assert:
		((mare eats: oats)
		and: [ doe eats: oats ]
		and: [ lamb eats: ivy ]
		and: [ kid eats: ivy ]
		).! !

TestCase subclass: #MetaclassHierarchyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!MetaclassHierarchyTest commentStamp: 'on 3/19/2006 11:33' prior: 0!
Demonstrate the relationship between classes and metaclasses!

!MetaclassHierarchyTest methodsFor: 'running' stamp: 'on 5/5/2006 16:56'!
testHierarchy
	"The class hierarchy"
	self assert: Snake superclass = Square.
	self assert: Square superclass = Object.
	self assert: Object superclass superclass = nil. "skip ProtoObject"

	"The parallel metaclass hierarchy"
	self assert: Snake class name = 'Snake class'.
	self assert: Snake class superclass = Square class.
	self assert: Square class superclass = Object class.
	self assert: Object class superclass superclass = Class.
	self assert: Class superclass = ClassDescription.
	self assert: ClassDescription superclass = Behavior.
	self assert: Behavior superclass = Object.

	"The Metaclass hierarchy"
	self assert: Snake class class = Metaclass.
	self assert: Square class class = Metaclass.
	self assert: Object class class = Metaclass.
	self assert: Class class class = Metaclass.
	self assert: ClassDescription class class = Metaclass.
	self assert: Behavior class class = Metaclass.
	self assert: Metaclass superclass = ClassDescription.

	"The fixpoint"
	self assert: Metaclass class class = Metaclass.
! !

TestCase subclass: #MinimalObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Reflection'!

!MinimalObjectTest methodsFor: 'running' stamp: 'on 4/3/2006 15:58'!
testCallCount
	"Transcript closeAllViews; open"
	| set minObject |
	"create subject and wrapper"
	set _ Set new.	
	minObject _ MinimalObject new.

	set add: 0.
	self assert: minObject MOcalls = 0.

	"swap the pointers"
	set become: minObject.
	set add: 1; add: 2.
	self assert: set MOcalls = 2.

	"swap the pointers"
	minObject become: set.
	set add: 3.
	self assert: minObject MOcalls = 2.
! !

TestCase subclass: #MwCountTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Reflection'!

!MwCountTest methodsFor: 'running' stamp: 'on 5/17/2006 14:47'!
testCounts
	| wrapper eg |
	wrapper := MwCountMethodWrapper on: #playOneMove inClass: SnakesAndLadders.
	wrapper install.
	eg := SnakesAndLadders example.
	self assert: wrapper count = 0.
	eg playOneMove; playOneMove; playOneMove.
	self assert: wrapper count = 3.
	wrapper uninstall.! !

TestCase subclass: #PostOfficeTest
	instanceVariableNames: 'postoffice'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-PostOffice'!

!PostOfficeTest methodsFor: 'running' stamp: 'on 6/15/2006 13:48'!
setUp
	postoffice := PostOffice new.! !

!PostOfficeTest methodsFor: 'running' stamp: 'on 6/15/2006 14:10'!
testPostOffice
	postoffice enter: 'jack'.
	self assert: postoffice waiting = 1.
	postoffice enter: 'jane'.
	self assert: postoffice waiting = 2.
	postoffice enter: 'jill'.
	self assert: postoffice waiting = 3.
	self assert: postoffice serve = 'jack'.
	self assert: postoffice waiting = 2.
	self assert: postoffice serve = 'jane'.
	self assert: postoffice waiting = 1.
	self assert: postoffice serve = 'jill'.
	self assert: postoffice waiting = 0.! !

TestCase subclass: #ReflectionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Reflection'!

!ReflectionTest commentStamp: 'on 3/24/2006 16:41' prior: 0!
Various demonstrations of reflective capabilities of Squeak.
Examples from Marcus Denker and Stphane Ducasse!

!ReflectionTest methodsFor: 'running' stamp: 'on 3/24/2006 16:52'!
testBecome
	| pt1 pt2 pt3 |
	pt1 := 0@0.
	pt2 := pt1.
	pt3 := 100@100.
	pt1 become: pt3.
	self assert: pt1 = (100@100).
	self assert: pt1 == pt2.
	self assert: pt3 = (0@0).! !

!ReflectionTest methodsFor: 'running' stamp: 'on 3/24/2006 16:51'!
testBecomeForward
	| pt1 pt2 pt3 |
	pt1 := 0@0.
	pt2 := pt1.
	pt3 := 100@100.
	pt1 becomeForward: pt3.
	self assert: pt1 = (100@100).
	self assert: pt1 == pt2.
	self assert: pt2 == pt3.
! !

!ReflectionTest methodsFor: 'running' stamp: 'on 3/24/2006 16:48'!
testPrimitiveChangeClassTo
	| behavior browser |
	behavior := Behavior new.
	behavior superclass: Browser.
	behavior setFormat: Browser format.
	browser := Browser new.
	browser primitiveChangeClassTo: behavior new.
	behavior compile: 'thisIsATest ^ 2'.
	self assert: browser thisIsATest = 2.
	self should: [Browser new thisIsATest] raise: MessageNotUnderstood.! !

TestCase subclass: #SnakesAndLaddersTest
	instanceVariableNames: 'eg loadedDie jack jill'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!SnakesAndLaddersTest commentStamp: 'on 3/15/2006 13:24' prior: 0!
Play a rigged game and check the results.!

!SnakesAndLaddersTest methodsFor: 'running' stamp: 'on 3/16/2006 09:58'!
example
	^ SnakesAndLadders example! !

!SnakesAndLaddersTest methodsFor: 'running' stamp: 'on 3/16/2006 09:58'!
setUp
	eg := self example.
	loadedDie := LoadedDie new.
	eg setDie: loadedDie.
	jack := eg players first.
	jill := eg players last.! !

!SnakesAndLaddersTest methodsFor: 'running' stamp: 'on 3/23/2006 09:06'!
testExample
	self assert: eg currentPlayer = jack.
	loadedDie roll: 1.
	eg playOneMove.
	self assert: jack position = 6.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 5.
	eg playOneMove.
	self assert: jill position = 1.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 5.
	eg playOneMove.
	self assert: jack position = 5.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 4.
	eg playOneMove.
	self assert: jill position = 1.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 6.
	eg playOneMove.
	self assert: jack position = 5.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 6.
	eg playOneMove.
	self assert: jill position = 9.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 5.
	eg playOneMove.
	self assert: jack position = 10.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 5.
	eg playOneMove.
	self assert: jill position = 1.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 2.
	eg playOneMove.
	self assert: jack position = 12.
	self assert: eg isOver! !

!SnakesAndLaddersTest methodsFor: 'running' stamp: 'on 3/18/2006 15:01'!
testStartPosition
	self assert: eg lastPosition = 12.
	self assert: eg isNotOver.
	self assert: eg currentPlayer = jack.
	self assert: eg firstSquare isFirstSquare.
	self assert: eg firstSquare isLastSquare not.
	self assert: eg firstSquare position = 1.
	self assert: eg firstSquare isOccupied.
	self assert: (eg at: eg lastPosition) isFirstSquare not.
	self assert: (eg at: eg lastPosition) isLastSquare.
	self assert: (eg at: eg lastPosition) position = 12.
	self assert: (eg at: eg lastPosition) isOccupied not.
	self assert: jack name = 'Jack'.
	self assert: jill name = 'Jill'.
	self assert: jack position = 1.
	self assert: jill position = 1! !

SnakesAndLaddersTest subclass: #SnakesAndLaddersTest2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!SnakesAndLaddersTest2 commentStamp: 'on 3/17/2006 10:50' prior: 0!
Some tests for a second example.!

!SnakesAndLaddersTest2 methodsFor: 'running' stamp: 'on 3/16/2006 09:59'!
example
	^ SnakesAndLadders example2! !

!SnakesAndLaddersTest2 methodsFor: 'running' stamp: 'on 3/18/2006 15:04'!
testExample
	self assert: jack name = 'Jack'.
	self assert: jill name = 'Jill'.
	self assert: jack position = 1.
	self assert: jill position = 1.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 1.
	eg playOneMove.
	self assert: jack position = 2.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 2.
	eg playOneMove.
	self assert: jill position = 7.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 3.
	eg playOneMove.
	self assert: jack position = 5.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 1.
	eg playOneMove.
	self assert: jill position = 7.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 2.
	eg playOneMove.
	self assert: jack position = 1.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 5.
	eg playOneMove.
	self assert: jill position = 12.
	self assert: eg currentPlayer = jack.
	loadedDie roll: 6.
	eg playOneMove.
	self assert: jack position = 7.
	self assert: eg currentPlayer = jill.
	loadedDie roll: 1.
	eg playOneMove.
	self assert: jill position = 13.
	self assert: eg isOver! !

Object subclass: #A
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SelfSuper'!

!A commentStamp: 'on 3/19/2006 16:20' prior: 0!
Toy example to illustrate how self and super interact.
Try:
A demo!

!A classSide methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:37'!
demo
	"self demo"
	Transcript closeAllViews; open.
	self withAllSubclasses do:
		[ :each |
			Transcript show: each asString , ' new foo --> ', each new foo; cr.
			Transcript show: each asString , ' new bar --> ', each new bar; cr.
			]! !

!A methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:35'!
bar
	^ 'Abar'! !

!A methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:25'!
foo
	^ 'Afoo'! !

A subclass: #B
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SelfSuper'!

!B methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:36'!
bar
	^ super bar, ' & ', self foo! !

B subclass: #C
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SelfSuper'!

!C methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:37'!
foo
	^ 'Cfoo'! !

C subclass: #D
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SelfSuper'!

!D methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:38'!
bar
	^ super bar, ' & ', super foo! !

D subclass: #E
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SelfSuper'!

!E methodsFor: 'as yet unclassified' stamp: 'on 3/16/2006 18:37'!
foo
	^ 'Efoo'! !

Object subclass: #Animal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Animal methodsFor: 'testing' stamp: 'on 3/14/2006 11:59'!
eats: aFood
	^ aFood isGoodFor: self! !

!Animal methodsFor: 'testing' stamp: 'on 3/14/2006 12:01'!
eatsIvy
	^ false! !

!Animal methodsFor: 'testing' stamp: 'on 3/14/2006 12:01'!
eatsOats
	^ false! !

Animal subclass: #Doe
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Doe methodsFor: 'testing' stamp: 'on 3/14/2006 12:01'!
eatsOats
	^ true! !

Animal subclass: #Kid
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Kid methodsFor: 'testing' stamp: 'on 3/14/2006 12:03'!
eatsIvy
	^ true! !

Animal subclass: #Lamb
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Lamb methodsFor: 'testing' stamp: 'on 3/14/2006 12:03'!
eatsIvy
	^ true! !

Animal subclass: #Mare
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Mare methodsFor: 'accessing' stamp: 'on 3/14/2006 12:03'!
eatsOats
	^ true! !

Object subclass: #Die
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!Die commentStamp: 'on 3/14/2006 19:06' prior: 0!
Responsibilities:
- Yields a random number from 1 to 6 ;-)!

!Die methodsFor: 'playing' stamp: 'on 3/14/2006 15:41'!
roll
	^ (1 to: 6) atRandom! !

Die subclass: #LoadedDie
	instanceVariableNames: 'roll'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!LoadedDie commentStamp: 'on 3/17/2006 10:50' prior: 0!
For testing purposes. When rolled, a loaded die returns a fixed value.!

!LoadedDie methodsFor: 'playing' stamp: 'on 3/18/2006 15:24'!
roll
	self assert: roll notNil.
	^ roll! !

!LoadedDie methodsFor: 'playing' stamp: 'on 3/18/2006 15:25'!
roll: aNumber
	self assert: ((1 to: 6) includes: aNumber).
	roll := aNumber.! !

Object subclass: #Fibs
	instanceVariableNames: 'fibCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Cacheing'!

!Fibs commentStamp: 'on 3/21/2006 19:06' prior: 0!
Fast, cacheing version of Fibonacci function.
Linear instead of exponential.

Fibs new at: 100
!

!Fibs methodsFor: 'public' stamp: 'on 3/21/2006 16:59'!
at: anIndex
	"Fibs new at: 30"
	self assert: anIndex >= 1.
	anIndex = 1 ifTrue: [ ^ 1 ].
	anIndex = 2 ifTrue: [ ^ 1 ].
	^ (self lookup: anIndex - 1) + (self lookup: anIndex - 2)! !

!Fibs methodsFor: 'private' stamp: 'on 3/21/2006 19:05'!
fibCache
	^ fibCache! !

!Fibs methodsFor: 'private' stamp: 'on 3/21/2006 19:05'!
initialize
	fibCache := Dictionary new! !

!Fibs methodsFor: 'private' stamp: 'on 3/21/2006 16:20'!
lookup: anIndex
	^ self fibCache at: anIndex ifAbsentPut: [ self at: anIndex ]! !

Object subclass: #Food
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Food methodsFor: 'testing' stamp: 'on 3/14/2006 12:00'!
isGoodFor: anAnimal
	^ self subclassResponsibility! !

Food subclass: #Ivy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Ivy methodsFor: 'testing' stamp: 'on 3/14/2006 12:00'!
isGoodFor: anAnimal
	^ anAnimal eatsIvy! !

Food subclass: #Oats
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-MaresyDoats'!

!Oats methodsFor: 'testing' stamp: 'on 3/14/2006 12:02'!
isGoodFor: anAnimal
	^ anAnimal eatsOats! !

Object subclass: #GamePlayer
	instanceVariableNames: 'name square'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!GamePlayer commentStamp: 'on 3/14/2006 19:07' prior: 0!
Responsibilities:
- Keep track of the square you are on
- Roll the die
- Attempt to move the resulting number of squares!

!GamePlayer classSide methodsFor: 'instance creation' stamp: 'on 3/18/2006 12:48'!
named: aName
	^ self new setName: aName! !

!GamePlayer methodsFor: 'initialize-release' stamp: 'on 3/18/2006 12:08'!
invariant
	^ square notNil
		and: [ name notNil ]! !

!GamePlayer methodsFor: 'private' stamp: 'on 3/14/2006 15:53'!
leaveCurrentSquare
	square notNil ifTrue: [ square remove: self ].! !

!GamePlayer methodsFor: 'playing' stamp: 'on 3/18/2006 15:21'!
moveTo: aSquare
	self leaveCurrentSquare.
	square := aSquare landHere: self.
! !

!GamePlayer methodsFor: 'playing' stamp: 'on 3/23/2006 09:04'!
moveWith: aDie
	| roll destination |
	self assert: self invariant.
	roll := aDie roll.
	destination := square forwardBy: roll.
	self moveTo: destination.
	^ name, ' rolls ', roll asString, ' and lands at ', square asString! !

!GamePlayer methodsFor: 'testing' stamp: 'on 3/15/2006 14:57'!
name
	^ name! !

!GamePlayer methodsFor: 'testing' stamp: 'on 3/14/2006 17:03'!
position
	^ square position! !

!GamePlayer methodsFor: 'printing' stamp: 'on 3/14/2006 15:32'!
printOn: aStream
	aStream nextPutAll: name! !

!GamePlayer methodsFor: 'initialize-release' stamp: 'on 3/18/2006 12:49'!
setName: aName
	name := aName.! !

Object subclass: #HaltDemo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Reflection'!

!HaltDemo commentStamp: 'on 3/28/2006 15:56' prior: 0!
HaltDemo new foo.
HaltDemo new bar.

Only the second halts within foo.!

!HaltDemo methodsFor: 'as yet unclassified' stamp: 'on 3/28/2006 15:01'!
bar
	^ (self foo), 'bar'! !

!HaltDemo methodsFor: 'as yet unclassified' stamp: 'on 3/28/2006 15:01'!
foo
	self haltIf: #bar.
	^ 'foo'! !

!HaltDemo methodsFor: 'as yet unclassified' stamp: 'on 3/28/2006 14:59'!
haltIf: aSelector
	| context |
	context := thisContext.
	[context sender isNil]
		whileFalse:
			[context := context sender.
			(context selector = aSelector)
			ifTrue: [ Halt signal ] ].
! !

Object subclass: #PostOffice
	instanceVariableNames: 'queue counters serving'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-PostOffice'!

!PostOffice classSide methodsFor: 'as yet unclassified' stamp: 'on 6/15/2006 15:45'!
demo
	"self demo"
	| postoffice |
	postoffice := self withCounters: 4.
	(1 to: 100) do: [ : each |
		2 atRandom = 1
			ifTrue:
				[ postoffice enter: 'customer ', each asString ]
			ifFalse:
				[ postoffice serve ]
			]! !

!PostOffice classSide methodsFor: 'as yet unclassified' stamp: 'on 6/15/2006 15:45'!
withCounters: aNumber
	| postoffice |
	postoffice := self new.
	postoffice counters: aNumber.
	^ postoffice! !

!PostOffice methodsFor: 'initialize-release' stamp: 'on 6/15/2006 15:44'!
counters: aNumber
	counters := aNumber! !

!PostOffice methodsFor: 'initialize-release' stamp: 'on 6/15/2006 14:39'!
defaultCounters
	^ 2! !

!PostOffice methodsFor: 'evaluating' stamp: 'on 6/15/2006 13:54'!
enter: aCustomer 
	queue addLast: aCustomer.! !

!PostOffice methodsFor: 'initialize-release' stamp: 'on 6/15/2006 14:39'!
initialize
	queue := OrderedCollection new.
	counters := self defaultCounters.
	serving := 1.! !

!PostOffice methodsFor: 'evaluating' stamp: 'on 6/15/2006 15:43'!
serve
	| customer |
	queue isEmpty ifTrue: [^ ''].
	customer := queue removeFirst.
	self serve: customer.
	self switchServing.
	^ customer! !

!PostOffice methodsFor: 'evaluating' stamp: 'on 6/15/2006 14:42'!
serve: customer
	Transcript
		show: 'Counter ' , serving asString , ' serving ' , customer;
		cr! !

!PostOffice methodsFor: 'evaluating' stamp: 'on 6/15/2006 13:53'!
switchServing
	serving := serving \\ counters + 1.! !

!PostOffice methodsFor: 'testing' stamp: 'on 6/15/2006 13:55'!
waiting
	^ queue size! !

Object subclass: #SlowFibs
	instanceVariableNames: 'fibCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Cacheing'!

!SlowFibs commentStamp: 'on 3/21/2006 16:56' prior: 0!
Slow, naive implementating of Fibonacci function.
Try:
SlowFibs new at: 30!

!SlowFibs methodsFor: 'as yet unclassified' stamp: 'on 3/21/2006 16:56'!
at: anIndex
	"SlowFibs new at: 30"
	self assert: anIndex >= 1.
	anIndex = 1 ifTrue: [ ^ 1 ].
	anIndex = 2 ifTrue: [ ^ 1 ].
	^ (self at: anIndex - 1) + (self at: anIndex - 2)! !

Object subclass: #SnakesAndLadders
	instanceVariableNames: 'players squares turn die over'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!SnakesAndLadders commentStamp: 'on 3/14/2006 19:05' prior: 0!
SnakesAndLadders example playToEnd

A simple board game in which players roll a die to move from the starting square to the end. A square may contain a snake (moves you back) or a ladder (moves you up). If youland on a square already occupied, you go back to the start. If you move past the end, you reverse direction the remaining number of moves.

See http://en.wikipedia.org/wiki/Snakes_and_ladders for the rules of the game

SnakesAndLadders has the following responsibilities:
- enable scripting of an initial configuration of squares and players
- keep track of the rules of the game
- keep track whose turn it is to play
- initiate a move
- play a game to the end
- report on moves and intermediate game states
- stop when the game is over

Typical usage (see the example):
- Instantiate a new SnakesAndLadders
- Add squares, snakes and ladders to build the game board 
- Join players
- Play single moves or PlayToEnd!

!SnakesAndLadders classSide methodsFor: 'documentation' stamp: 'on 3/16/2006 09:55'!
example
	"self example playToEnd"
	^ self new
		add: FirstSquare new;
		add: (Ladder forward: 4);
		add: Square new;
		add: Square new;
		add: Square new;
		add: Square new;
		add: (Ladder forward: 2);
		add: Square new;
		add: Square new;
		add: Square new;
		add: (Snake back: 6);
		add: Square new;
		join: (GamePlayer named: 'Jack');
		join: (GamePlayer named: 'Jill');
		yourself! !

!SnakesAndLadders classSide methodsFor: 'documentation' stamp: 'on 3/16/2006 12:02'!
example2
	"self example2 playToEnd"
	^ self new
		add: FirstSquare new;
		add: Square new;
		add: (Ladder forward: 4);
		add: Square new;
		add: Square new;
		add: (Snake back: 2);
		add: Square new;
		add: (Ladder forward: 3);
		add: Square new;
		add: Square new;
		add: (Snake back: 4);
		add: Square new;
		add: Square new;
		join: (GamePlayer named: 'Jack');
		join: (GamePlayer named: 'Jill');
		yourself! !

!SnakesAndLadders methodsFor: 'initialize-release' stamp: 'on 3/18/2006 15:13'!
add: aSquare 
	squares add: aSquare.
	aSquare setPosition: squares size board: self! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/14/2006 16:30'!
at: position
	^ squares at: position! !

!SnakesAndLadders methodsFor: 'playing' stamp: 'on 3/14/2006 17:10'!
checkResult
	^ self currentPlayer position = self lastPosition
		ifTrue: [
			over := true.
			' -- ', self currentPlayer asString, ' wins!!'
			]
		ifFalse: [ '' ]! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/14/2006 16:54'!
currentPlayer
	^ players at: turn! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/14/2006 16:43'!
firstSquare
	^ squares first! !

!SnakesAndLadders methodsFor: 'initialize-release' stamp: 'on 3/20/2006 10:16'!
initialize
	super initialize.
	die := Die new.
	squares := OrderedCollection new.
	players := OrderedCollection new.
	turn := 1.
	over := false.! !

!SnakesAndLadders methodsFor: 'initialize-release' stamp: 'on 3/14/2006 19:12'!
invariant
	"Should also check that snakes and ladders lead to ordinary squares, and do not bounce
	past the beginning or end of the board."
	^ squares size > 1
		and: [players size > 1]
		and: [turn >= 1]
		and: [turn <= players size]! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/18/2006 15:04'!
isNotOver
	^ self isOver not! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/18/2006 15:04'!
isOver
	^ over! !

!SnakesAndLadders methodsFor: 'initialize-release' stamp: 'on 3/18/2006 12:10'!
join: aPlayer
	players add: aPlayer.
	aPlayer moveTo: self firstSquare.
! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/14/2006 16:59'!
lastPosition
	^ squares size! !

!SnakesAndLadders methodsFor: 'playing' stamp: 'on 5/17/2006 16:49'!
playOneMove
	| result |
	self assert: self invariant.
	^ self isOver 
		ifTrue: ['The game is over!!']
		ifFalse: 
			[result := (self currentPlayer moveWith: die) , self checkResult.
			self upDateTurn.
			result]! !

!SnakesAndLadders methodsFor: 'playing' stamp: 'on 3/18/2006 15:01'!
playToEnd
	"SnakesAndLadders example playToEnd"

	Transcript closeAllViews.
	Transcript open.
	Transcript
		show: self asString;
		cr;
		cr.
	[self isNotOver] whileTrue: 
			[Transcript
				show: self playOneMove asString;
				cr.
			Transcript
				show: self asString;
				cr;
				cr]! !

!SnakesAndLadders methodsFor: 'accessors' stamp: 'on 3/15/2006 15:01'!
players
	"needed for testing"
	^ players! !

!SnakesAndLadders methodsFor: 'printing' stamp: 'on 3/16/2006 18:06'!
printOn: aStream
	squares do: [:each | each printOn: aStream].! !

!SnakesAndLadders methodsFor: 'initialize-release' stamp: 'on 3/16/2006 11:30'!
reset
	die := Die new.
	turn := 1.
	over := false.
	players do: [:each | each moveTo: self firstSquare ].
	self assert: self invariant.! !

!SnakesAndLadders methodsFor: 'accessors' stamp: 'on 3/15/2006 13:29'!
setDie: aDie
	"For testing only. Can play the game with a loaded die."
	die := aDie.! !

!SnakesAndLadders methodsFor: 'testing' stamp: 'on 3/14/2006 16:22'!
turn
	^ turn! !

!SnakesAndLadders methodsFor: 'playing' stamp: 'on 3/14/2006 16:21'!
upDateTurn
	turn := 1 + (turn\\players size).! !

Object subclass: #Square
	instanceVariableNames: 'position player board'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!Square commentStamp: 'on 3/14/2006 19:27' prior: 0!
Responsibilities:
- Remember position in the board
- Keep track of any players on this square
- Send on any players passing through
- Displays itself with its position and any player who might be on it, e.g., [7 Jack]!

Square subclass: #FirstSquare
	instanceVariableNames: 'players'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!FirstSquare commentStamp: 'on 3/14/2006 19:06' prior: 0!
Responsibilities:
- Allows multiple players to be added or removed [normal squares can only hold a single player]!

!FirstSquare methodsFor: 'printing' stamp: 'on 3/14/2006 15:35'!
contents
	^ players inject: '' into:
			[ :list :each | list , ' ', each asString ]! !

!FirstSquare methodsFor: 'initialize-release' stamp: 'on 3/14/2006 15:30'!
initialize
	players := OrderedCollection new.! !

!FirstSquare methodsFor: 'testing' stamp: 'on 3/18/2006 11:10'!
isOccupied
	^ players size > 0! !

!FirstSquare methodsFor: 'initialize-release' stamp: 'on 3/15/2006 15:29'!
landHere: aPlayer
	players add: aPlayer.! !

!FirstSquare methodsFor: 'playing' stamp: 'on 3/14/2006 16:15'!
remove: aPlayer
	self assert: (players includes: aPlayer).
	players remove: aPlayer.! !

Square subclass: #Ladder
	instanceVariableNames: 'forward'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!Ladder commentStamp: 'on 3/17/2006 10:36' prior: 0!
Responsibilities:
- If a player lands here, send him or her up the ladder (forward # squares)
- Displays itself as a square with a positive offset, e.g., [6]3+>!

!Ladder classSide methodsFor: 'instance creation' stamp: 'on 3/18/2006 12:50'!
forward: number
	^ self new setForward: number! !

!Ladder methodsFor: 'playing' stamp: 'on 3/15/2006 15:26'!
destination
	^ self forwardBy: forward! !

!Ladder methodsFor: 'printing' stamp: 'on 3/16/2006 14:35'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: forward asString, '+>'.! !

!Ladder methodsFor: 'initialize-release' stamp: 'on 3/18/2006 12:49'!
setForward: aNumber
	forward := aNumber.! !

Square subclass: #Snake
	instanceVariableNames: 'back'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-SnakesAndLadders'!

!Snake commentStamp: 'on 3/20/2006 09:14' prior: 0!
Responsibilities:
- If a player lands here, send him or her backwards (back # squares)
- Displays itself as a square with a negative offset, e.g., <-3[5]!

!Snake classSide methodsFor: 'instance creation' stamp: 'on 3/18/2006 12:50'!
back: number
	^ self new setBack: number! !

!Snake methodsFor: 'playing' stamp: 'on 3/15/2006 15:26'!
destination
	^ self backwardBy: back! !

!Snake methodsFor: 'printing' stamp: 'on 3/16/2006 14:35'!
printOn: aStream
	aStream nextPutAll: '<-', back asString.
	super printOn: aStream.! !

!Snake methodsFor: 'initialize-release' stamp: 'on 3/18/2006 12:50'!
setBack: aNumber
	back := aNumber.! !

!Square methodsFor: 'playing' stamp: 'on 3/15/2006 15:26'!
backwardBy: squares 
	^ squares = 0 
		ifTrue: [self destination]
		ifFalse: 
			[self isFirstSquare 
				ifTrue: [self nextSquare forwardBy: squares - 1]
				ifFalse: [self previousSquare backwardBy: squares - 1]]! !

!Square methodsFor: 'printing' stamp: 'on 3/14/2006 17:18'!
contents
	^ self isOccupied
		ifTrue: [ ' ', player asString ]
		ifFalse: [ '' ]! !

!Square methodsFor: 'playing' stamp: 'on 3/15/2006 15:26'!
destination
	^ self! !

!Square methodsFor: 'playing' stamp: 'on 3/18/2006 11:43'!
forwardBy: squares
	self assert: self invariant.
	^ squares = 0 
		ifTrue: [self destination]
		ifFalse: 
			[self isLastSquare 
				ifFalse: [self nextSquare forwardBy: squares - 1]
				ifTrue: [self previousSquare backwardBy: squares - 1]]! !

!Square methodsFor: 'initialize-release' stamp: 'on 3/18/2006 11:42'!
invariant
	^ board notNil
		and: [ position notNil ]
		and: [ position >= 0 ]
		and: [ position <= board lastPosition]! !

!Square methodsFor: 'testing' stamp: 'on 3/14/2006 16:12'!
isFirstSquare
	^ position = 1! !

!Square methodsFor: 'testing' stamp: 'on 3/20/2006 10:17'!
isLastSquare
	^ position = board lastPosition! !

!Square methodsFor: 'testing' stamp: 'on 3/14/2006 16:40'!
isOccupied
	^ player notNil! !

!Square methodsFor: 'playing' stamp: 'on 3/15/2006 15:43'!
landHere: aPlayer
	^ self isOccupied
		ifTrue: [ board firstSquare landHere: aPlayer ]
		ifFalse: [ player := aPlayer. self ]! !

!Square methodsFor: 'playing' stamp: 'on 3/14/2006 16:32'!
nextSquare
	self assert: self isLastSquare not.
	^ board at: position + 1! !

!Square methodsFor: 'testing' stamp: 'on 3/14/2006 17:03'!
position
	^ position! !

!Square methodsFor: 'playing' stamp: 'on 3/14/2006 16:32'!
previousSquare
	self assert: self isFirstSquare not.
	^ board at: position - 1! !

!Square methodsFor: 'printing' stamp: 'on 3/14/2006 16:08'!
printOn: aStream
	aStream nextPutAll: '[', position printString, self contents, ']'.! !

!Square methodsFor: 'playing' stamp: 'on 3/14/2006 16:14'!
remove: aPlayer
	self assert: player = aPlayer.
	player := nil.! !

!Square methodsFor: 'initialize-release' stamp: 'on 3/18/2006 15:13'!
setPosition: aNumber board: aBoard 
	position := aNumber.
	board := aBoard! !

ProtoObject subclass: #MinimalObject
	instanceVariableNames: 'subject calls'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SqueakExamples-Reflection'!

!MinimalObject commentStamp: 'on 3/28/2006 16:27' prior: 0!
A MinimalObject wraps a subject and intercepts all messages sent to it.
!

!MinimalObject classSide methodsFor: 'as yet unclassified' stamp: 'on 3/30/2006 15:39'!
demo
	"Transcript closeAllViews; open.
	MinimalObject demo"
	| set minObject |
	"create subject and wrapper"
	set := Set new.	
	minObject := MinimalObject new.

	set add: 0.

	"swap the pointers"
	set become: minObject.
	set add: 1; add: 2.

	"swap the pointers"
	minObject become: set.
	set add: 3.
	set inspect.
! !

!MinimalObject methodsFor: 'as yet unclassified' stamp: 'on 4/3/2006 15:18'!
MOcalls
	"Choose an uncommon method name"
	^ calls! !

!MinimalObject methodsFor: 'as yet unclassified' stamp: 'on 4/3/2006 13:57'!
doesNotUnderstand: aMessage
	Transcript show: 'performing ', aMessage printString; cr.
	calls := calls + 1.
	^ subject perform: aMessage selector withArguments: aMessage arguments! !

!MinimalObject methodsFor: 'as yet unclassified' stamp: 'on 4/3/2006 13:57'!
initialize
	"When we become: anotherObject, this reference will also get swapped"
	subject _ self.
	calls := 0.! !
